package com.example.ellegadodepintia

import android.animation.ObjectAnimator
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.animation.doOnEnd
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.repositorios.RepositorioJugador
import com.example.ellegadodepintia.soundManager.SoundManager
import com.google.firebase.auth.FirebaseAuth
import kotlin.random.Random

class AuthActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_register)

        val botonRegistrarse = findViewById<Button>(R.id.buttonRegister)
        val botonIniciarSesion = findViewById<Button>(R.id.buttonLogIn)
        val textoEmail = findViewById<TextView>(R.id.editTextEmail).text
        val textoPassword = findViewById<TextView>(R.id.editTextPassword).text
        val textoUsername = findViewById<TextView>(R.id.editTextUsername).text
        var soundManager: SoundManager
        soundManager = SoundManager(this)
        // Verificar si hay un usuario autenticado
        val currentUser = FirebaseAuth.getInstance().currentUser
        if (currentUser != null) {
            RepositorioJugador.cargarJugador(currentUser.email!!)
            lluviaDeNubes()
            soundManager.playSound(R.raw.sound_login,100)
            val titleTextView = findViewById<TextView>(R.id.titleTextView)
            titleTextView.text = "CARGANDO MUNDO"
            botonRegistrarse.visibility = View.GONE
            botonIniciarSesion.visibility = View.GONE
            Handler(Looper.getMainLooper()).postDelayed({
                mostrarMapActivity()
            }, 2000)
            return
        }



        botonRegistrarse.setOnClickListener {
            if (textoEmail.isNotEmpty() && textoPassword.isNotEmpty() && textoUsername.isNotEmpty()) {
                FirebaseAuth.getInstance().createUserWithEmailAndPassword(
                    textoEmail.toString(),
                    textoPassword.toString()
                ).addOnCompleteListener {
                    if (it.isSuccessful) {
                        RepositorioJugador.addJugador(textoEmail.toString(), textoUsername.toString())
                        RepositorioJugador.cargarJugador(textoEmail.toString())
                        lluviaDeNubes()
                        val titleTextView = findViewById<TextView>(R.id.titleTextView)
                        titleTextView.text = "CARGANDO MUNDO"
                        botonRegistrarse.visibility = View.GONE
                        botonIniciarSesion.visibility = View.GONE
                        Handler(Looper.getMainLooper()).postDelayed({
                            mostrarMapActivity()
                        }, 2000)
                    } else {
                        showAlert()
                    }
                }
            } else {
                showAlert()
            }
        }

        botonIniciarSesion.setOnClickListener {
            if (textoEmail.isNotEmpty() && textoPassword.isNotEmpty()) {
                FirebaseAuth.getInstance().signInWithEmailAndPassword(
                    textoEmail.toString(),
                    textoPassword.toString()
                ).addOnCompleteListener {
                    if (it.isSuccessful) {
                        RepositorioJugador.cargarJugador(textoEmail.toString())
                        lluviaDeNubes()
                        soundManager.playSound(R.raw.sound_login,100)
                        val titleTextView = findViewById<TextView>(R.id.titleTextView)
                        titleTextView.text = "CARGANDO MUNDO"
                        botonRegistrarse.visibility = View.GONE
                        botonIniciarSesion.visibility = View.GONE
                        Handler(Looper.getMainLooper()).postDelayed({
                            mostrarMapActivity()
                        }, 2000)
                    } else {
                        showAlert()
                    }
                }
            }
        }
    }

    private fun showAlert() {
        val builder = AlertDialog.Builder(this)
        builder.setTitle("Error")
        builder.setMessage("Se ha producido un error en la autenticación")
        builder.setPositiveButton("Aceptar", null)
        val dialog: AlertDialog = builder.create()
        dialog.show()
    }

    private fun mostrarMapActivity() {
        val intent = Intent(this, MapActivity::class.java)
        startActivity(intent)
        finish() // Finaliza la actividad actual para que no vuelva atrás
    }


    // Inicia el efecto de lluvia de nubes
    private fun lluviaDeNubes() {
        val screenWidth = resources.displayMetrics.widthPixels
        val screenHeight = resources.displayMetrics.heightPixels
        val rainDuration = 2000L  // Duración de la lluvia de nubes
        val handler = Handler(Looper.getMainLooper())
        val startTime = System.currentTimeMillis()

        // Runnable para generar las nubes mientras dure la lluvia
        val runnable = object : Runnable {
            override fun run() {
                val elapsedTime = System.currentTimeMillis() - startTime
                if (elapsedTime < rainDuration) {
                    nubeCaer(screenWidth, screenHeight)
                    handler.postDelayed(this, 30)  // Cada 100ms cae una nueva nube
                }
            }
        }
        handler.post(runnable)
    }

    // Crea una nube animada que cae desde la parte superior de la pantalla
    private fun nubeCaer(screenWidth: Int, screenHeight: Int) {
        val nube = ImageView(this).apply {
            setImageResource(R.drawable.asi_minijuego_2_nube)  // Imagen de la nube
            layoutParams = ViewGroup.LayoutParams(200, 200)  // Tamaño de la nube
            x = Random.nextInt(0, screenWidth - 100).toFloat()  // Posición X aleatoria
            y = -100f  // Empieza fuera de la pantalla (parte superior)
        }

        // Añadimos la nube al layout actual
        addContentView(nube, nube.layoutParams)

        // Animación de caída de la nube
        ObjectAnimator.ofFloat(nube, "translationY", screenHeight.toFloat() + 100f).apply {
            duration = Random.nextLong(1000, 3000)  // Duración aleatoria para cada nube
            start()

            // Cuando la animación termine, se elimina la nube del layout
            doOnEnd { (nube.parent as? ViewGroup)?.removeView(nube) }
        }
    }


}
