package com.example.ellegadodepintia.exploradoresDePintia.model

import android.app.Activity
import android.content.Context
import android.view.View
import android.widget.ImageView
import android.widget.ProgressBar
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.fragment.app.FragmentManager
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.ModalDetallesFragment
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.Objeto
import android.animation.ObjectAnimator
import com.example.ellegadodepintia.soundManager.SoundManager
object LayoutUtils {

    private lateinit var soundManager: SoundManager
    fun actualizarEstadisticasMain() {
        if (context is Activity) {
            val barraVida = context!!.findViewById<ProgressBar>(R.id.barraVida)
            val barraEnergia = context!!.findViewById<ProgressBar>(R.id.barraEnergia)
            val ataqueText = context!!.findViewById<TextView>(R.id.ataqueText)
            val investigacionText = context!!.findViewById<TextView>(R.id.investigacionText)
            val vidaText = context!!.findViewById<TextView>(R.id.vidaText)
            val energiaText = context!!.findViewById<TextView>(R.id.EnergiaText)

            actualizarJugador(barraVida, ataqueText, investigacionText, vidaText, energiaText, barraEnergia,true)
        }
    }

    fun actualizarEstadisticasInventario(view: View) {
        val barraVida = view.findViewById<ProgressBar>(R.id.barraVida)
        val barraEnergia = context!!.findViewById<ProgressBar>(R.id.barraEnergia)
        val ataqueText = view.findViewById<TextView>(R.id.ataqueText)
        val investigacionText = view.findViewById<TextView>(R.id.investigacionText)
        val vidaText = view.findViewById<TextView>(R.id.vidaText)
        val energiaText = view.findViewById<TextView>(R.id.EnergiaText)
        actualizarJugador(barraVida, ataqueText, investigacionText, vidaText, energiaText, barraEnergia,false)
    }



    private fun actualizarJugador(
        barraVida: ProgressBar,
        ataqueText: TextView,
        investigacionText: TextView,
        vidaText: TextView,
        energiaText: TextView,
        barraEnergia: ProgressBar,
        sonido: Boolean
    ) {
        val vidaMax = GameState.jugador.vidaMax
        val vidaActual = GameState.jugador.atributos[Atributo.Vida]!!
        soundManager = SoundManager(context!!)
        if(barraVida.progress>vidaActual && vidaActual!=vidaMax && sonido){
            soundManager.playSound(R.raw.sound_dano,100)
        }
        if (barraVida.progress != vidaActual) {
            ObjectAnimator.ofInt(barraVida, "progress", barraVida.progress, vidaActual).apply {
                duration = 500
                start()
            }
        }
        barraVida.max = vidaMax
        ataqueText.text = "${GameState.jugador.atributos[Atributo.Ataque]}"
        investigacionText.text = "${GameState.jugador.atributos[Atributo.Investigacion]}"
        val vidaInfo = "$vidaActual/$vidaMax"
        vidaText.text = vidaInfo
        val energiaMax = GameState.jugador.energiaMax
        val energiaActual = GameState.jugador.atributos[Atributo.Energia]!!
        if (barraEnergia.progress != energiaActual) {
            ObjectAnimator.ofInt(barraEnergia, "progress", barraEnergia.progress, energiaActual).apply {
                duration = 500
                start()
            }
        }
        barraEnergia.max = energiaMax
        val energiaInfo = "$energiaActual/$energiaMax"
        energiaText.text = energiaInfo
    }

    fun actualizarEstadisticasEvento() {
        val barraEvento = context!!.findViewById<ProgressBar>(R.id.barraEvento)
        barraEvento.max = GameState.eventoActual.resistenciaMax
        barraEvento.progress = GameState.eventoActual.resistenciaActual
    }

    fun actualizarInventarioUI(view: View, parentFragmentManager: FragmentManager) {
        val slots = listOf(
            view.findViewById(R.id.slot01),
            view.findViewById(R.id.slot02),
            view.findViewById(R.id.slot03),
            view.findViewById(R.id.slot04),
            view.findViewById(R.id.slot05),
            view.findViewById(R.id.slot06),
            view.findViewById(R.id.slot07),
            view.findViewById(R.id.slot08),
            view.findViewById<CardView>(R.id.slot09)
        )

        val slotsEquipable = listOf(
            view.findViewById(R.id.slot1),
            view.findViewById(R.id.slot2),
            view.findViewById<CardView>(R.id.slot3)
        )

        actualizarSlots(slots, GameState.jugador.inventario, view, parentFragmentManager)
        actualizarSlots(slotsEquipable, GameState.jugador.equipables, view, parentFragmentManager)
    }

    private fun actualizarSlots(slots: List<View>, items: List<Objeto>, view: View, parentFragmentManager: FragmentManager) {
        for (i in slots.indices) {
            val slot = slots[i] as CardView
            if (i < items.size) {
                val imageView = slot.getChildAt(0) as ImageView
                imageView.setImageResource(items[i].imagen)
                slot.getChildAt(0).visibility = View.VISIBLE

                slot.setOnClickListener {
                    val modalDetallesFragment = ModalDetallesFragment()
                    modalDetallesFragment.objeto = items[i]
                    modalDetallesFragment.parentView = view
                    modalDetallesFragment.show(parentFragmentManager, "modalDetalles")
                }
            } else {
                slot.getChildAt(0).visibility = View.GONE
                slot.setOnClickListener(null)
            }
        }
    }

    fun actualizarMonedas() {
        GameState.jugador.monedas.toString()
            .also { context!!.findViewById<TextView>(R.id.moneyText).text = it }
    }

    fun actualizarMonedasTienda(tienda : Context) {
        GameState.jugador.monedas.toString()
            .also { (tienda as Activity).findViewById<TextView>(R.id.textoMonedas).text = it }
    }

    fun actualizarMonedasInventario(view: View) {
        GameState.jugador.monedas.toString()
            .also { view.findViewById<TextView>(R.id.moneyText).text = it }
    }


}